package org.jeecg.modules.ai.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ai.entity.PreReviewMaterial;
import org.jeecg.modules.ai.qo.PreReviewMaterialQuery;
import org.jeecg.modules.ai.service.IPreReviewMaterialService;
import org.jeecg.modules.ai.vo.PreReviewMaterialVo;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 通用预审材料
 * @Author: jeecg-boot
 * @Date:   2023-11-25
 * @Version: V1.0
 */
@Api(tags="通用预审材料")
@RestController
@RequestMapping("/ai/preReviewMaterial")
@Slf4j
public class PreReviewMaterialController extends JeecgController<PreReviewMaterial, IPreReviewMaterialService> {
	@Autowired
	private IPreReviewMaterialService preReviewMaterialService;
	
	/**
	 * 分页列表查询
	 *
	 * @param preReviewMaterial
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "通用预审材料-分页列表查询")
	@ApiOperation(value="通用预审材料-分页列表查询", notes="通用预审材料-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(PreReviewMaterialQuery preReviewMaterialQuery,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		Page<PreReviewMaterialVo> page = new Page<PreReviewMaterialVo>(pageNo, pageSize);
		IPage<PreReviewMaterialVo> pageList = preReviewMaterialService.getPreReviewMaterialPage(page, preReviewMaterialQuery);
		return Result.OK(pageList);
	}
	
	/**
	 * 列表查询
	 *
	 * @param preReviewMaterial
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "通用预审材料-列表查询")
	@ApiOperation(value="通用预审材料-列表查询", notes="通用预审材料-列表查询")
	@GetMapping(value = "/queryList")
	public Result<?> queryList(PreReviewMaterialQuery preReviewMaterialQuery,
			HttpServletRequest req) {
		List<PreReviewMaterialVo> pageList = preReviewMaterialService.getPreReviewMaterialList(preReviewMaterialQuery.getEventCode());
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param preReviewMaterial
	 * @return
	 */
	@AutoLog(value = "通用预审材料-添加")
	@ApiOperation(value="通用预审材料-添加", notes="通用预审材料-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody PreReviewMaterialQuery preReviewMaterialQuery) {
		preReviewMaterialService.saveInfo(preReviewMaterialQuery.getIds());
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param preReviewMaterial
	 * @return
	 */
	@AutoLog(value = "通用预审材料-编辑")
	@ApiOperation(value="通用预审材料-编辑", notes="通用预审材料-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody PreReviewMaterial preReviewMaterial) {
		preReviewMaterialService.updateById(preReviewMaterial);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "通用预审材料-通过id删除")
	@ApiOperation(value="通用预审材料-通过id删除", notes="通用预审材料-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		preReviewMaterialService.removeById(id);
		return Result.OK("删除成功!");
	}
	/**
	 *   通过id启用或停用
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "通用预审材料-通过id启用或停用")
	@ApiOperation(value="通用预审材料-通过id启用或停用", notes="通用预审材料-通过id启用或停用")
	@GetMapping(value = "/cancel")
	public Result<?> editStatus(@RequestParam(name="ids",required=true) String ids) {
		List<String> idsList = Arrays.asList(ids.split(","));
		List<PreReviewMaterial> preReviewMaterialOldList = preReviewMaterialService.listByIds(idsList);
		preReviewMaterialOldList.stream().forEach(preReviewMaterial -> {
			preReviewMaterial.setValidStatus(preReviewMaterial.getValidStatus().intValue()==0?1:0);
        });
		preReviewMaterialService.updateBatchById(preReviewMaterialOldList);
		return Result.OK("操作成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "通用预审材料-批量删除")
	@ApiOperation(value="通用预审材料-批量删除", notes="通用预审材料-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.preReviewMaterialService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "通用预审材料-通过id查询")
	@ApiOperation(value="通用预审材料-通过id查询", notes="通用预审材料-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		PreReviewMaterial preReviewMaterial = preReviewMaterialService.getById(id);
		if(preReviewMaterial==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(preReviewMaterial);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param preReviewMaterial
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PreReviewMaterial preReviewMaterial) {
        return super.exportXls(request, preReviewMaterial, PreReviewMaterial.class, "通用预审材料");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PreReviewMaterial.class);
    }

}
