package org.jeecg.modules.ai.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ai.entity.PreReviewRecords;
import org.jeecg.modules.ai.qo.PreReviewRecordsQuery;
import org.jeecg.modules.ai.service.IPreReviewRecordsService;
import org.jeecg.modules.ai.vo.PreReviewRecordsVo;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 办事预审记录
 * @Author: jeecg-boot
 * @Date:   2023-11-25
 * @Version: V1.0
 */
@Api(tags="办事预审记录")
@RestController
@RequestMapping("/ai/preReviewRecords")
@Slf4j
public class PreReviewRecordsController extends JeecgController<PreReviewRecords, IPreReviewRecordsService> {
	@Autowired
	private IPreReviewRecordsService preReviewRecordsService;
	
	/**
	 * 分页列表查询
	 *
	 * @param preReviewRecords
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "办事预审记录-分页列表查询")
	@ApiOperation(value="办事预审记录-分页列表查询", notes="办事预审记录-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(PreReviewRecordsQuery preReviewRecordsQuery,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		Page<PreReviewRecordsVo> page = new Page<PreReviewRecordsVo>(pageNo, pageSize);
		IPage<PreReviewRecordsVo> pageList = preReviewRecordsService.getPreReviewRecordsList(page, preReviewRecordsQuery);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param preReviewRecords
	 * @return
	 */
	@AutoLog(value = "办事预审记录-添加")
	@ApiOperation(value="办事预审记录-添加", notes="办事预审记录-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody String reviewInfo) {
		return Result.OK(preReviewRecordsService.saveInfo(reviewInfo));
	}
	
	/**
	 *  编辑
	 *
	 * @param preReviewRecords
	 * @return
	 */
	@AutoLog(value = "办事预审记录-编辑")
	@ApiOperation(value="办事预审记录-编辑", notes="办事预审记录-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody PreReviewRecords preReviewRecords) {
		preReviewRecordsService.updateById(preReviewRecords);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "办事预审记录-通过id删除")
	@ApiOperation(value="办事预审记录-通过id删除", notes="办事预审记录-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		preReviewRecordsService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "办事预审记录-批量删除")
	@ApiOperation(value="办事预审记录-批量删除", notes="办事预审记录-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.preReviewRecordsService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id下载用户上传文件
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "办事预审记录-通过id下载用户上传文件")
	@ApiOperation(value="办事预审记录-通过id下载用户上传文件", notes="办事预审记录-通过id下载用户上传文件")
	@GetMapping(value = "/downLoad")
	public Result<?> downLoad(@RequestParam(name="id",required=true) String id) {
		PreReviewRecords preReviewRecords = preReviewRecordsService.getById(id);
		if(preReviewRecords==null) {
			return Result.error("未找到对应数据");
		}
		String path = preReviewRecordsService.downLoad(preReviewRecords);
		return Result.OK(preReviewRecords);
	}
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "办事预审记录-通过id查询")
	@ApiOperation(value="办事预审记录-通过id查询", notes="办事预审记录-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		PreReviewRecords preReviewRecords = preReviewRecordsService.getById(id);
		if(preReviewRecords==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(preReviewRecords);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param preReviewRecords
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PreReviewRecords preReviewRecords) {
        return super.exportXls(request, preReviewRecords, PreReviewRecords.class, "办事预审记录");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PreReviewRecords.class);
    }
    

}
