package org.jeecg.modules.ai.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ai.entity.PreReviewRecordsDetail;
import org.jeecg.modules.ai.qo.PreReviewRecordsDetailQuery;
import org.jeecg.modules.ai.service.IPreReviewRecordsDetailService;
import org.jeecg.modules.ai.vo.PreReviewRecordsDetailVo;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 办事预审记录详情
 * @Author: jeecg-boot
 * @Date:   2023-11-25
 * @Version: V1.0
 */
@Api(tags="办事预审记录详情")
@RestController
@RequestMapping("/ai/preReviewRecordsDetail")
@Slf4j
public class PreReviewRecordsDetailController extends JeecgController<PreReviewRecordsDetail, IPreReviewRecordsDetailService> {
	@Autowired
	private IPreReviewRecordsDetailService preReviewRecordsDetailService;
	
	/**
	 * 分页列表查询
	 *
	 * @param preReviewRecordsDetail
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-分页列表查询")
	@ApiOperation(value="办事预审记录详情-分页列表查询", notes="办事预审记录详情-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(PreReviewRecordsDetailQuery preReviewRecordsDetailQuery,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		Page<PreReviewRecordsDetailVo> page = new Page<PreReviewRecordsDetailVo>(pageNo, pageSize);
		IPage<PreReviewRecordsDetailVo> pageList = preReviewRecordsDetailService.getPreReviewRecordsDetailList(page, preReviewRecordsDetailQuery);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param preReviewRecordsDetail
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-添加")
	@ApiOperation(value="办事预审记录详情-添加", notes="办事预审记录详情-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody PreReviewRecordsDetail preReviewRecordsDetail) {
		preReviewRecordsDetailService.save(preReviewRecordsDetail);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param preReviewRecordsDetail
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-编辑")
	@ApiOperation(value="办事预审记录详情-编辑", notes="办事预审记录详情-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody PreReviewRecordsDetail preReviewRecordsDetail) {
		preReviewRecordsDetailService.updateById(preReviewRecordsDetail);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-通过id删除")
	@ApiOperation(value="办事预审记录详情-通过id删除", notes="办事预审记录详情-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		preReviewRecordsDetailService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-批量删除")
	@ApiOperation(value="办事预审记录详情-批量删除", notes="办事预审记录详情-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.preReviewRecordsDetailService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "办事预审记录详情-通过id查询")
	@ApiOperation(value="办事预审记录详情-通过id查询", notes="办事预审记录详情-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		PreReviewRecordsDetail preReviewRecordsDetail = preReviewRecordsDetailService.getById(id);
		if(preReviewRecordsDetail==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(preReviewRecordsDetail);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param preReviewRecordsDetail
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PreReviewRecordsDetail preReviewRecordsDetail) {
        return super.exportXls(request, preReviewRecordsDetail, PreReviewRecordsDetail.class, "办事预审记录详情");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PreReviewRecordsDetail.class);
    }

}
