package org.jeecg.modules.ai.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ai.entity.YsDirectory;
import org.jeecg.modules.ai.qo.YsDirectoryQuery;
import org.jeecg.modules.ai.service.IYsDirectoryService;
import org.jeecg.modules.ai.vo.YsDirectoryVo;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 事项表
 * @Author: jeecg-boot
 * @Date:   2023-11-25
 * @Version: V1.0
 */
@Api(tags="事项表")
@RestController
@RequestMapping("/ai/ysDirectory")
@Slf4j
public class YsDirectoryController extends JeecgController<YsDirectory, IYsDirectoryService> {
	@Autowired
	private IYsDirectoryService ysDirectoryService;
	
	/**
	 * 分页列表查询
	 *
	 * @param ysDirectory
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "事项表-分页列表查询")
	@ApiOperation(value="事项表-分页列表查询", notes="事项表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(YsDirectory ysDirectory,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<YsDirectory> queryWrapper = QueryGenerator.initQueryWrapper(ysDirectory, req.getParameterMap());
		Page<YsDirectory> page = new Page<YsDirectory>(pageNo, pageSize);
		IPage<YsDirectory> pageList = ysDirectoryService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 * 分页列表查询(多表)
	 *
	 * @param ysDirectory
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "事项表-分页列表查询(多表)")
	@ApiOperation(value="事项表-分页列表查询(多表)", notes="事项表-分页列表查询(多表)")
	@GetMapping(value = "/page")
	public Result<?> getYsDirectoryList(YsDirectoryQuery queryWrapper,
			@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
			@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
			HttpServletRequest req) {
		Page<YsDirectoryVo> page = new Page<YsDirectoryVo>(pageNo, pageSize);
		IPage<YsDirectoryVo> pageList = ysDirectoryService.getYsDirectoryList(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param ysDirectory
	 * @return
	 */
	@AutoLog(value = "事项表-添加")
	@ApiOperation(value="事项表-添加", notes="事项表-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody YsDirectory ysDirectory) {
		ysDirectoryService.save(ysDirectory);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param ysDirectory
	 * @return
	 */
	@AutoLog(value = "事项表-编辑")
	@ApiOperation(value="事项表-编辑", notes="事项表-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody YsDirectory ysDirectory) {
		ysDirectoryService.updateById(ysDirectory);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "事项表-通过id删除")
	@ApiOperation(value="事项表-通过id删除", notes="事项表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		ysDirectoryService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "事项表-批量删除")
	@ApiOperation(value="事项表-批量删除", notes="事项表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.ysDirectoryService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "事项表-通过id查询")
	@ApiOperation(value="事项表-通过id查询", notes="事项表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		YsDirectory ysDirectory = ysDirectoryService.getById(id);
		if(ysDirectory==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(ysDirectory);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param ysDirectory
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, YsDirectory ysDirectory) {
        return super.exportXls(request, ysDirectory, YsDirectory.class, "事项表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, YsDirectory.class);
    }

}
