package org.jeecg.modules.ai.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ai.entity.YsDirectoryMaterial;
import org.jeecg.modules.ai.service.IYsDirectoryMaterialService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 资料表
 * @Author: jeecg-boot
 * @Date:   2023-11-25
 * @Version: V1.0
 */
@Api(tags="资料表")
@RestController
@RequestMapping("/ai/ysDirectoryMaterial")
@Slf4j
public class YsDirectoryMaterialController extends JeecgController<YsDirectoryMaterial, IYsDirectoryMaterialService> {
	@Autowired
	private IYsDirectoryMaterialService ysDirectoryMaterialService;
	
	/**
	 * 分页列表查询
	 *
	 * @param ysDirectoryMaterial
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "资料表-分页列表查询")
	@ApiOperation(value="资料表-分页列表查询", notes="资料表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(YsDirectoryMaterial ysDirectoryMaterial,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<YsDirectoryMaterial> queryWrapper = QueryGenerator.initQueryWrapper(ysDirectoryMaterial, req.getParameterMap());
		Page<YsDirectoryMaterial> page = new Page<YsDirectoryMaterial>(pageNo, pageSize);
		IPage<YsDirectoryMaterial> pageList = ysDirectoryMaterialService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param ysDirectoryMaterial
	 * @return
	 */
	@AutoLog(value = "资料表-添加")
	@ApiOperation(value="资料表-添加", notes="资料表-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody YsDirectoryMaterial ysDirectoryMaterial) {
		ysDirectoryMaterialService.save(ysDirectoryMaterial);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param ysDirectoryMaterial
	 * @return
	 */
	@AutoLog(value = "资料表-编辑")
	@ApiOperation(value="资料表-编辑", notes="资料表-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody YsDirectoryMaterial ysDirectoryMaterial) {
		ysDirectoryMaterialService.updateById(ysDirectoryMaterial);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "资料表-通过id删除")
	@ApiOperation(value="资料表-通过id删除", notes="资料表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		ysDirectoryMaterialService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "资料表-批量删除")
	@ApiOperation(value="资料表-批量删除", notes="资料表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.ysDirectoryMaterialService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "资料表-通过id查询")
	@ApiOperation(value="资料表-通过id查询", notes="资料表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		YsDirectoryMaterial ysDirectoryMaterial = ysDirectoryMaterialService.getById(id);
		if(ysDirectoryMaterial==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(ysDirectoryMaterial);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param ysDirectoryMaterial
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, YsDirectoryMaterial ysDirectoryMaterial) {
        return super.exportXls(request, ysDirectoryMaterial, YsDirectoryMaterial.class, "资料表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, YsDirectoryMaterial.class);
    }

}
