package org.jeecg.modules.system.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.system.entity.MaterialBag;
import org.jeecg.modules.system.service.IMaterialBagService;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 你好
 * @Author: jeecg-boot
 * @Date:   2023-11-20
 * @Version: V1.0
 */
@Api(tags="material_bag")
@RestController
@RequestMapping("/system/materialBag")
@Slf4j
public class MaterialBagController extends JeecgController<MaterialBag, IMaterialBagService> {
	@Autowired
	private IMaterialBagService materialBagService;
	
	/**
	 * 分页列表查询
	 *
	 * @param materialBag
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "材料袋-分页列表查询")
	@ApiOperation(value="材料袋-分页列表查询", notes="材料袋-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(MaterialBag materialBag,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<MaterialBag> queryWrapper = QueryGenerator.initQueryWrapper(materialBag, req.getParameterMap());
		Page<MaterialBag> page = new Page<MaterialBag>(pageNo, pageSize);
		IPage<MaterialBag> pageList = materialBagService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param materialBag
	 * @return
	 */
	@AutoLog(value = "你好-添加")
	@ApiOperation(value="你好-添加", notes="你好-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody MaterialBag materialBag) {
	    materialBag.setBagQrCode(materialBag.getCode());
		materialBagService.save(materialBag);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param materialBag
	 * @return
	 */
	@AutoLog(value = "你好-编辑")
	@ApiOperation(value="你好-编辑", notes="你好-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody MaterialBag materialBag) {
		materialBagService.updateById(materialBag);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "你好-通过id删除")
	@ApiOperation(value="你好-通过id删除", notes="你好-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		materialBagService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "你好-批量删除")
	@ApiOperation(value="你好-批量删除", notes="你好-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.materialBagService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "你好-通过id查询")
	@ApiOperation(value="你好-通过id查询", notes="你好-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		MaterialBag materialBag = materialBagService.getById(id);
		if(materialBag==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(materialBag);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param materialBag
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, MaterialBag materialBag) {
        return super.exportXls(request, materialBag, MaterialBag.class, "你好");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, MaterialBag.class);
    }


	 /**
	  * 启动/暂停
	  *
	  * @param ids
	  * @return
	  */
	 @AutoLog(value = "资料袋-作废")
	 @ApiOperation(value = "资料袋-作废", notes = "资料袋-作废")
	 @GetMapping(value = "/cancel")
	 @Transactional
	 public Result<?> editStatus(@RequestParam(name = "ids") String ids) {
		 List<String> idsList = Arrays.asList(ids.split(","));
//		 //判断传入的id是否有误
//		 Boolean validateId = datasSiteService.validateIds(idsList);
//		 if (!validateId) {
//			 return Result.error(CommonConstant.SC_INTERNAL_SERVER_ERROR_400, "ids有误");
//		 }
//
//		 if (status != 0 && status != 1) {
//			 return Result.error(CommonConstant.SC_INTERNAL_SERVER_ERROR_400, "status有误!");
//		 }

		 materialBagService.cancel(idsList);
		 return Result.OK("作废成功!");
	 }

}
