package org.jeecg.modules.system.controller;

import java.io.IOException;
import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.authz.annotation.RequiresRoles;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.constant.CacheConstant;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.util.JwtUtil;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.ImportExcelUtil;
import org.jeecg.common.util.YouBianCodeUtil;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.constant.SendChangeMsgConstant;
import org.jeecg.modules.system.entity.*;
import org.jeecg.modules.system.enums.DepartEnum;
import org.jeecg.modules.system.model.DepartIdModel;
import org.jeecg.modules.system.model.SysDepartTreeModel;
import org.jeecg.modules.system.service.*;
import org.jeecg.modules.system.to.SysDepartTo;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 部门表 前端控制器
 * <p>
 *
 * @Author: Steve @Since： 2019-01-22
 */
@Api(tags = "组织机构管理")
@RestController
@RequestMapping("/sys/sysDepart")
@Slf4j
public class SysDepartController {
    @Autowired
    private ISysChangeMsgService sysChangeMsgService;
    @Autowired
    private ISysDepartService sysDepartService;
    @Autowired
    public RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ISysUserDepartService sysUserDepartService;
    @Autowired
    private ISysAreaService sysAreaService;
    @Autowired
    private ISysTenantService tenantService;

    /**
     * 查询数据 查出我的部门,并以树结构数据格式响应给前端
     *
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-分页列表查询", notes = "gt_sys_depart-分页列表查询")
    @RequestMapping(value = "/queryMyDeptTreeList", method = RequestMethod.GET)
    @RequiresPermissions("system/DeptList_lock")
    public Result<List<SysDepartTreeModel>> queryMyDeptTreeList() {
        Result<List<SysDepartTreeModel>> result = new Result<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        try {
            if (oConvertUtils.isNotEmpty(user.getUserIdentity()) && user.getUserIdentity().equals(CommonConstant.USER_IDENTITY_2)) {
                List<SysDepartTreeModel> list = sysDepartService.queryMyDeptTreeList(user.getDepartIds());
                result.setResult(list);
                result.setMessage(CommonConstant.USER_IDENTITY_2.toString());
                result.setSuccess(true);
            } else {
                result.setMessage(CommonConstant.USER_IDENTITY_1.toString());
                result.setSuccess(true);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }


    @AutoLog(value = "sys_area-全部条件查询")
    @ApiOperation(value = "sys_area-全部条件查询", notes = "sys_area-全部条件查询")
    @RequiresPermissions("system/DeptList_lock")
    @GetMapping(value = "/queryList")
    public Result<?> queryList() {
        List<SysArea> entities = sysAreaService.listWithTreeByNameAndCode();
        //List<SysArea> list = sysAreaService.list();
        return Result.OK(entities);
    }

    /**
     * 查询数据 查出所有部门,并以树结构数据格式响应给前端
     *
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-树结构查询", notes = "gt_sys_depart-树结构查询")
    @RequestMapping(value = "/queryTreeList", method = RequestMethod.GET)
//    @RequiresPermissions("system/DeptList_lock")
    public Result<List<SysDepartTreeModel>> queryTreeList() {
        Result<List<SysDepartTreeModel>> result = new Result<>();
        try {
            // 从内存中读取
//			List<SysDepartTreeModel> list =FindsDepartsChildrenUtil.getSysDepartTreeList();
//			if (CollectionUtils.isEmpty(list)) {
//				list = sysDepartService.queryTreeList();
//			}
            List<SysDepartTreeModel> list = sysDepartService.queryTreeList();
            result.setResult(list);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }
    /**
     * 查询数据 查出所有部门,并以树结构数据格式响应给前端
     *
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-树结构查询测试", notes = "gt_sys_depart-树结构查询测试")
    @RequestMapping(value = "/queryTreeListTest", method = RequestMethod.GET)
//    @RequiresPermissions("system/DeptList_lock")
    public Result<List<SysDepartTreeModel>> queryTreeListTest() {
    	Result<List<SysDepartTreeModel>> result = new Result<>();
    	try {
    		// 从内存中读取
//			List<SysDepartTreeModel> list =FindsDepartsChildrenUtil.getSysDepartTreeList();
//			if (CollectionUtils.isEmpty(list)) {
//				list = sysDepartService.queryTreeList();
//			}
    		List<SysDepartTreeModel> list = sysDepartService.queryTreeListTest();
    		result.setResult(list);
    		result.setSuccess(true);
    	} catch (Exception e) {
    		log.error(e.getMessage(), e);
    	}
    	return result;
    }

//	@ApiOperation(value="gt_sys_depart-areaId查询", notes="gt_sys_depart-areaId查询")
//	@RequestMapping(value = "/queryTreeListByAreaLevel", method = RequestMethod.GET)
//	public Result<List<SysDepartTreeModel>> queryTreeListByAreaId(@RequestParam("areaId") String areaId) {
//		Result<List<SysDepartTreeModel>> result = new Result<>();
//
//		List<String> pidByAreaId = sysDepartService.getPidByAreaId(areaId);
//		//List<SysDepart> sysDeparts = sysDepartService.listByIds(pidByAreaId);
//		List<SysDepartTreeModel> list =sysDepartService.queryTreeListByAreaId(areaId);
//		result.setResult(list);
//		result.setSuccess(true);
//		return result;
//	}

    @ApiOperation(value = "areaId查询", notes = "areaId查询")
    @RequestMapping(value = "/queryByAreaId", method = RequestMethod.GET)
    public Result<List<SysDepart>> queryByAreaId(@RequestParam("areaId") String areaId) {
        Result<List<SysDepart>> result = new Result<>();

        List<SysDepart> list = sysDepartService.getDeptByAreaId(areaId);
        result.setResult(list);
        result.setSuccess(true);
        return result;
    }

    @ApiOperation(value = "areaId查询", notes = "areaId查询")
    @RequestMapping(value = "/queryValidDeptByAreaId", method = RequestMethod.GET)
    public Result<List<SysDepart>> queryValidDeptByAreaId(@RequestParam("areaId") String areaId) {
        Result<List<SysDepart>> result = new Result<>();
        List<SysDepart> list = sysDepartService.getValidDeptByAreaId(areaId);
        result.setResult(list);
        result.setSuccess(true);
        return result;
    }


    @ApiOperation(value = "gt_sys_depart-areaId查询", notes = "gt_sys_depart-areaId查询")
    @RequestMapping(value = "/queryTreeListByAreaId", method = RequestMethod.GET)
    public Result<List<SysDepartTreeModel>> queryTreeListByAreaId(@RequestParam("areaId") String areaId) {
        Result<List<SysDepartTreeModel>> result = new Result<>();

        //List<String> pidByAreaId = sysDepartService.getPidByAreaId(areaId);
        //List<SysDepart> sysDeparts = sysDepartService.listByIds(pidByAreaId);
        List<SysDepartTreeModel> list = sysDepartService.queryTreeListByAreaId(areaId);
        result.setResult(list);
        result.setSuccess(true);
        return result;
    }


    /**
     * 异步查询部门list
     *
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-异步查询部门list", notes = "gt_sys_depart-异步查询部门list")
    @RequestMapping(value = "/queryDepartTreeSync", method = RequestMethod.GET)
    public Result<List<SysDepartTreeModel>> queryDepartTreeSync(@RequestParam(name = "pid", required = false) String parentId) {
        Result<List<SysDepartTreeModel>> result = new Result<>();
        try {
            List<SysDepartTreeModel> list = sysDepartService.queryTreeListByPid(parentId);
            result.setResult(list);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }


    /**
     * 添加新数据 添加用户新建的部门对象数据,并保存到数据库
     *
     * @param sysDepart
     * @return
     */
    //@RequiresRoles({"admin"})
    @AutoLog(value = "sys_dept-添加新数据")
    @ApiOperation(value = "gt_sys_depart-添加新数据", notes = "gt_sys_depart-添加新数据")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    @CacheEvict(value = {CacheConstant.SYS_DEPARTS_CACHE, CacheConstant.SYS_DEPART_IDS_CACHE}, allEntries = true)
    @Transactional
    public Result<?> add(@RequestBody SysDepart sysDepart, HttpServletRequest request) {
        Result<SysDepart> result = new Result<>();
        String username = JwtUtil.getUserNameByToken(request);
        try {
            sysDepart.setCreateBy("四川省一体化政务服务平台");
            sysDepart.setManubrium(0);
            String areaId = sysDepart.getAreaId();
            SysArea sysArea = sysAreaService.getById(areaId);
            sysDepart.setAreaCode(sysArea.getAreaCode());
            sysDepart.setAreaName(sysArea.getAreaName());
            sysDepart.setAreaLevel(sysArea.getAreaLevel());
            if (sysDepart.getParentId() != null) {
                String[] path = sysDepartService.findParentIdPath(sysDepart.getParentId());
                sysDepart.setBigDepartId(path[0]);
                SysDepart sysDepart1 = sysDepartService.getById(path[0]);
                sysDepart.setBigDepartCode(sysDepart1.getDepartCode());
                sysDepart.setBigDepartName(sysDepart1.getDepartName());
                String s = Arrays.toString(path);
                sysDepart.setDepartPath(s);
                sysDepart.setTreeLevel(path.length + "");
            } else {
                String id = String.valueOf(IdWorker.getId());
                sysDepart.setId(id);
                sysDepart.setBigDepartCode(sysDepart.getDepartCode());
                sysDepart.setBigDepartName(sysDepart.getDepartName());
                sysDepart.setBigDepartId(id);
                sysDepart.setDepartPath(sysDepart.getId());
                sysDepart.setTreeLevel("0");
            }
            SysDepart one = sysDepartService.getOne(new QueryWrapper<SysDepart>().eq("social_Credit_Code",
                    sysDepart.getSocialCreditCode()).eq("status", "1").last("limit 1"));
            if (!oConvertUtils.isEmpty(one)) {
                return Result.error(DepartEnum.DEPART_CODE17003.getCode(), DepartEnum.DEPART_CODE17003.getMsg());
            }
            if (oConvertUtils.isNotEmpty(sysDepart.getDepartCode())) {
                int count = sysDepartService.count(new QueryWrapper<SysDepart>().lambda().eq(SysDepart::getDepartCode, sysDepart.getDepartCode()).eq(SysDepart::getStatus, "1"));
                if (count > 0) {
                    return Result.error(DepartEnum.DEPART_CODE17005.getCode(), DepartEnum.DEPART_CODE17005.getMsg());
                }
            }

            sysDepartService.saveDepartData(sysDepart, username);
            tenantService.sendDeptMsg(sysDepart, SendChangeMsgConstant.INSERT);
            //清除部门树内存
            // FindsDepartsChildrenUtil.clearSysDepartTreeList();
            // FindsDepartsChildrenUtil.clearDepartIdModel();
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑数据 编辑部门的部分数据,并保存到数据库
     *
     * @param sysDepart
     * @return
     */
    //@RequiresRoles({"admin"})
    @AutoLog(value = "sys_dept-编辑数据")
    @ApiOperation(value = "gt_sys_depart-编辑数据 ", notes = "gt_sys_depart-编辑数据 ")
    @RequestMapping(value = "/edit", method = RequestMethod.PUT)
    @CacheEvict(value = {CacheConstant.SYS_DEPARTS_CACHE, CacheConstant.SYS_DEPART_IDS_CACHE}, allEntries = true)
    @Transactional
    public Result<?> edit(@RequestBody SysDepart sysDepart, HttpServletRequest request) {
        String username = JwtUtil.getUserNameByToken(request);
        sysDepart.setUpdateBy(username);
        Result<SysDepart> result = new Result<>();
        SysDepart sysDepartEntity = sysDepartService.getById(sysDepart.getId());
        if (sysDepartEntity == null) {
            result.error500("未找到对应实体");
        } else {
            if ("0".equals(sysDepart.getStatus()) && "1".equals(sysDepartEntity.getStatus())) {
                tenantService.sendDeptByDeptChange(sysDepart);
            }
            if (!"0".equals(sysDepart.getStatus())) {
                SysDepart one = sysDepartService.getOne(new QueryWrapper<SysDepart>().eq("social_Credit_Code", sysDepart.getSocialCreditCode()).eq("status", "1").last("limit 1"));
                if (!oConvertUtils.isEmpty(one) && !sysDepartEntity.getSocialCreditCode().equals(one.getSocialCreditCode())) {
                    return Result.error(DepartEnum.DEPART_CODE17003.getCode(), DepartEnum.DEPART_CODE17003.getMsg());
                    //return result.error10003("统⼀社会信⽤代码被占⽤");
                }
                if (!oConvertUtils.isEmpty(one) && !sysDepart.getId().equals(one.getId())) {
                    return Result.error(DepartEnum.DEPART_CODE17003.getCode(), DepartEnum.DEPART_CODE17003.getMsg());
                    //return result.error10003("统⼀社会信⽤代码被占⽤");
                }
                if (oConvertUtils.isNotEmpty(sysDepart.getDepartCode())) {
                    int count = sysDepartService.count(new QueryWrapper<SysDepart>().lambda().eq(SysDepart::getDepartCode, sysDepart.getDepartCode()).eq(SysDepart::getStatus, "1"));
                    SysDepart depart = sysDepartService.getOne(new QueryWrapper<SysDepart>().lambda().eq(SysDepart::getDepartCode, sysDepart.getDepartCode()).eq(SysDepart::getStatus, "1"));
                    if (count > 0 && !depart.getId().equals(sysDepart.getId())) {
                        return Result.error(DepartEnum.DEPART_CODE17005.getCode(), DepartEnum.DEPART_CODE17005.getMsg());
                        //return result.error10005("部门编码被占用");
                    }
                }
            }
            if (!sysDepartEntity.getDepartName().equals(sysDepart.getDepartName()) || sysDepartEntity.getDepartCode() != sysDepart.getDepartCode()) {
                List<SysDepart> departList = sysDepartService.list(new QueryWrapper<SysDepart>().eq("parent_id", sysDepart.getId()));
                if (departList != null && !departList.isEmpty()) {
                    departList.forEach(list -> {
                        list.setParentName(sysDepart.getDepartName());
                        list.setBigDepartName(sysDepart.getDepartName());
                        list.setBigDepartCode(sysDepart.getDepartCode());
                        SysDepartTo sysDepartTo = new SysDepartTo();
                        BeanUtils.copyProperties(list, sysDepartTo);
                        List<SysTenant> tenants = tenantService.list(new QueryWrapper<SysTenant>().isNotNull("msg_url").eq("is_send", "1"));
                        tenants.forEach(sysTenant -> {
                            String json = JSON.toJSONString(sysDepartTo);
                            sysChangeMsgService.saveChangeMsg(sysTenant.getId(), sysTenant.getMsgUrl(), json, SendChangeMsgConstant.UPDATE, SendChangeMsgConstant.TYPE_DEPT);
                        });
                        boolean ok = sysDepartService.updateDepartDataById(list, username);
                    });
                }
                if (oConvertUtils.isEmpty(sysDepart.getParentId())) {
                    sysDepart.setBigDepartCode(sysDepart.getDepartCode());
                    sysDepart.setBigDepartName(sysDepart.getDepartName());
                }
            }
            if (!sysDepartEntity.getDepartName().equals(sysDepart.getDepartName())) {
                List<SysUser> sysUsers = sysUserDepartService.queryUserByDepId(sysDepart.getId());
                if (sysUsers != null && !sysUsers.isEmpty()) {
                    sysUsers.forEach(user -> {
                        user.setDepartName(sysDepart.getDepartName());
                        user.setUpdateTime(new Date());
                        user.setUpdateBy(username);
                        tenantService.sendUserMsg(user, sysDepart, SendChangeMsgConstant.UPDATE);
                        boolean ok = sysUserService.updateById(user);
                    });
                }
            }
            boolean ok = sysDepartService.updateDepartDataById(sysDepart, username);
            // TODO 返回false说明什么？
            if (ok) {
                //清除部门树内存
                //FindsDepartsChildrenUtil.clearSysDepartTreeList();
                //FindsDepartsChildrenUtil.clearDepartIdModel();
                result.success("修改成功!");
            }

            SysDepartTo sysDepartTo = new SysDepartTo();
            BeanUtils.copyProperties(sysDepart, sysDepartTo);
            if (oConvertUtils.isEmpty(sysDepartTo.getBigDepartCode())) {
                sysDepartTo.setBigDepartCode("");
            }
            if (oConvertUtils.isEmpty(sysDepartTo.getBigDepartName())) {
                sysDepartTo.setBigDepartName("");
            }
            if (oConvertUtils.isEmpty(sysDepartTo.getDepartCode())) {
                sysDepartTo.setDepartCode("");
            }
            sysDepartTo.setUpdateTime(new Date());
            List<SysTenant> list = tenantService.list(new QueryWrapper<SysTenant>().isNotNull("msg_url").eq("is_send", "1"));
            list.forEach(sysTenant -> {
                String json = JSON.toJSONString(sysDepartTo);
                sysChangeMsgService.saveChangeMsg(sysTenant.getId(), sysTenant.getMsgUrl(), json, SendChangeMsgConstant.UPDATE, SendChangeMsgConstant.TYPE_DEPT);
            });
        }
        return result;
    }


    @RequestMapping(value = "/getLoginUserDept", method = RequestMethod.GET)
    public Result<SysDepart> queryUserAndArea() {
        Result<SysDepart> result = new Result<SysDepart>();
        LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        SysUser userById = sysUserService.getUserById(loginUser.getId());
        if (!oConvertUtils.isEmpty(userById)) {
            SysUserDepart sysUserDepart = sysUserDepartService.getbyUserId(userById.getId());
            SysDepart byId = sysDepartService.getById(sysUserDepart.getDepId());
            result.setResult(byId);
            return result;
        }
        return null;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    //@RequiresRoles({"admin"})
    @ApiOperation(value = "gt_sys_depart-id删除 ", notes = "gt_sys_depart-id删除 ")
    @AutoLog(value = "sys_dept-删除数据")
    @RequestMapping(value = "/delete", method = RequestMethod.DELETE)
    @CacheEvict(value = {CacheConstant.SYS_DEPARTS_CACHE, CacheConstant.SYS_DEPART_IDS_CACHE}, allEntries = true)
    public Result<SysDepart> delete(@RequestParam(name = "id", required = true) String id) {

        Result<SysDepart> result = new Result<SysDepart>();
        SysDepart sysDepart = sysDepartService.getById(id);
        if (sysDepart == null) {
            result.error500("未找到对应实体");
        } else {
            List<SysTenant> list = tenantService.list(new QueryWrapper<SysTenant>().isNotNull("msg_url").eq("is_send", "1"));
            list.forEach(sysTenant -> {
                String json = JSON.toJSONString(sysDepart);
                sysChangeMsgService.saveChangeMsg(sysTenant.getId(), sysTenant.getMsgUrl(), json, SendChangeMsgConstant.DELETE, SendChangeMsgConstant.TYPE_DEPT);
            });
            boolean ok = sysDepartService.delete(id);
            if (ok) {
                //清除部门树内存
                //FindsDepartsChildrenUtil.clearSysDepartTreeList();
                // FindsDepartsChildrenUtil.clearDepartIdModel();
                result.success("删除成功!");
            }
        }
        return result;
    }


    /**
     * 批量删除 根据前端请求的多个ID,对数据库执行删除相关部门数据的操作
     *
     * @param ids
     * @return
     */
    //@RequiresRoles({"admin"})
    @ApiOperation(value = "gt_sys_depart-批量删除 ", notes = "gt_sys_depart-批量删除 ")
    @AutoLog(value = "sys_area-批量删除")
    @RequestMapping(value = "/deleteBatch", method = RequestMethod.DELETE)
    @CacheEvict(value = {CacheConstant.SYS_DEPARTS_CACHE, CacheConstant.SYS_DEPART_IDS_CACHE}, allEntries = true)
    public Result<SysDepart> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {

        Result<SysDepart> result = new Result<SysDepart>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.sysDepartService.deleteBatchWithChildren(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 查询数据 添加或编辑页面对该方法发起请求,以树结构形式加载所有部门的名称,方便用户的操作
     *
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-查询数据 ", notes = "gt_sys_depart-查询数据 ")
    @RequestMapping(value = "/queryIdTree", method = RequestMethod.GET)
    public Result<List<DepartIdModel>> queryIdTree() {
//		Result<List<DepartIdModel>> result = new Result<List<DepartIdModel>>();
//		List<DepartIdModel> idList;
//		try {
//			idList = FindsDepartsChildrenUtil.wrapDepartIdModel();
//			if (idList != null && idList.size() > 0) {
//				result.setResult(idList);
//				result.setSuccess(true);
//			} else {
//				sysDepartService.queryTreeList();
//				idList = FindsDepartsChildrenUtil.wrapDepartIdModel();
//				result.setResult(idList);
//				result.setSuccess(true);
//			}
//			return result;
//		} catch (Exception e) {
//			log.error(e.getMessage(),e);
//			result.setSuccess(false);
//			return result;
//		}
        Result<List<DepartIdModel>> result = new Result<>();
        try {
            List<DepartIdModel> list = sysDepartService.queryDepartIdTreeList();
            result.setResult(list);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }

    @AutoLog(value = "gt_sys_depart-通过id查询子元素")
    @ApiOperation(value = "gt_sys_depart-通过id查询子元素", notes = "gt_sys_depart-通过id查询子元素")
    @GetMapping(value = "/queryChildById")
    public Result<?> queryChildById(@RequestParam(name = "id", required = true) String id) {
        List<SysDepart> entities = sysDepartService.listWithTreeById(id);
        return Result.OK(entities);
    }


    /**
     * <p>
     * 部门搜索功能方法,根据关键字模糊搜索相关部门
     * </p>
     *
     * @param keyWord
     * @return
     */
    @ApiOperation(value = "gt_sys_depart-关键字模糊搜索 ", notes = "gt_sys_depart-关键字模糊搜索 ")
    @RequestMapping(value = "/searchBy", method = RequestMethod.GET)
    public Result<List<SysDepartTreeModel>> searchBy(@RequestParam(name = "keyWord", required = true) String keyWord, @RequestParam(name = "myDeptSearch", required = false) String myDeptSearch) {
        Result<List<SysDepartTreeModel>> result = new Result<List<SysDepartTreeModel>>();
        //部门查询，myDeptSearch为1时为我的部门查询，登录用户为上级时查只查负责部门下数据
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String departIds = null;
        if (oConvertUtils.isNotEmpty(user.getUserIdentity()) && user.getUserIdentity().equals(CommonConstant.USER_IDENTITY_2)) {
            departIds = user.getDepartIds();
        }
        List<SysDepartTreeModel> treeList = this.sysDepartService.searhBy(keyWord, myDeptSearch, departIds);
        if (treeList == null || treeList.size() == 0) {
            result.setSuccess(false);
            result.setMessage("未查询匹配数据！");
            return result;
        }
        result.setResult(treeList);
        return result;
    }

    /**
     * 通过阻止机构名称和统一社会信用代码查询
     *
     * @param departName SOCIAL_CREDIT_CODE
     * @return
     */
    @AutoLog(value = "gt_sys_depart-通过参数查询")
    @ApiOperation(value = "gt_sys_depart-通过参数查询", notes = "gt_sys_depart-通过参数查询")
    @GetMapping(value = "/queryByNameAndCode")
    public Result<?> queryByname(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                 String departName, String socialCreditCode, String areaId) {
        IPage<SysDepart> list = sysDepartService.queryPage(pageNo, pageSize, departName, socialCreditCode, areaId);
        return Result.OK(list);
    }

    /**
     * 通过部门名称查询
     *
     * @param departSimpleName
     * @return
     */
    @AutoLog(value = "gt_sys_depart-通过参数查询")
    @ApiOperation(value = "gt_sys_depart-通过参数查询", notes = "gt_sys_depart-通过参数查询")
    @GetMapping(value = "/queryDeptByName")
    public Result<?> queryDeptByName(String departSimpleName) {
        List<SysDepart> list = sysDepartService.queryDeptList(departSimpleName);
        return Result.OK(list);
    }


    /**
     * 导出excel
     *
     * @param request
     */
    @AutoLog(value = "sys_area-导出excel")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(SysDepart sysDepart, HttpServletRequest request) {
        // Step.1 组装查询条件
        QueryWrapper<SysDepart> queryWrapper = QueryGenerator.initQueryWrapper(sysDepart, request.getParameterMap());
        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<SysDepart> pageList = sysDepartService.list(queryWrapper);
        //按字典排序
        Collections.sort(pageList, new Comparator<SysDepart>() {
            @Override
            public int compare(SysDepart arg0, SysDepart arg1) {
                return arg0.getOrgCode().compareTo(arg1.getOrgCode());
            }
        });
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "部门列表");
        mv.addObject(NormalExcelConstants.CLASS, SysDepart.class);
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("部门列表数据", "导出人:" + user.getRealname(), "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresRoles({"admin"})
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    @AutoLog(value = "sys_area-导入数据")
    @CacheEvict(value = {CacheConstant.SYS_DEPARTS_CACHE, CacheConstant.SYS_DEPART_IDS_CACHE}, allEntries = true)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<String> errorMessageList = new ArrayList<>();
        List<SysDepart> listSysDeparts = null;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                // orgCode编码长度
                int codeLength = YouBianCodeUtil.zhanweiLength;
                listSysDeparts = ExcelImportUtil.importExcel(file.getInputStream(), SysDepart.class, params);
                //按长度排序
                Collections.sort(listSysDeparts, new Comparator<SysDepart>() {
                    @Override
                    public int compare(SysDepart arg0, SysDepart arg1) {
                        return arg0.getOrgCode().length() - arg1.getOrgCode().length();
                    }
                });

                int num = 0;
                for (SysDepart sysDepart : listSysDeparts) {
                    String orgCode = sysDepart.getOrgCode();
                    if (orgCode.length() > codeLength) {
                        String parentCode = orgCode.substring(0, orgCode.length() - codeLength);
                        QueryWrapper<SysDepart> queryWrapper = new QueryWrapper<SysDepart>();
                        queryWrapper.eq("org_code", parentCode);
                        try {
                            SysDepart parentDept = sysDepartService.getOne(queryWrapper);
                            if (!parentDept.equals(null)) {
                                sysDepart.setParentId(parentDept.getId());
                            } else {
                                sysDepart.setParentId("");
                            }
                        } catch (Exception e) {
                            //没有查找到parentDept
                        }
                    } else {
                        sysDepart.setParentId("");
                    }
                    //update-begin---author:liusq   Date:20210223  for：批量导入部门以后，不能追加下一级部门 #2245------------
                    sysDepart.setOrgType(sysDepart.getOrgCode().length() / codeLength + "");
                    //update-end---author:liusq   Date:20210223  for：批量导入部门以后，不能追加下一级部门 #2245------------
                    sysDepart.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
                    ImportExcelUtil.importDateSaveOne(sysDepart, ISysDepartService.class, errorMessageList, num, CommonConstant.SQL_INDEX_UNIQ_DEPART_ORG_CODE);
                    num++;
                }
                //清空部门缓存
                Set keys3 = redisTemplate.keys(CacheConstant.SYS_DEPARTS_CACHE + "*");
                Set keys4 = redisTemplate.keys(CacheConstant.SYS_DEPART_IDS_CACHE + "*");
                redisTemplate.delete(keys3);
                redisTemplate.delete(keys4);
                return ImportExcelUtil.imporReturnRes(errorMessageList.size(), listSysDeparts.size() - errorMessageList.size(), errorMessageList);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.error("文件导入失败！");
    }


    /**
     * 查询所有部门信息
     *
     * @return
     */

    @GetMapping("listAll")
    public Result<List<SysDepart>> listAll(@RequestParam(name = "id", required = false) String id) {
        Result<List<SysDepart>> result = new Result<>();
        LambdaQueryWrapper<SysDepart> query = new LambdaQueryWrapper<SysDepart>();
        query.orderByAsc(SysDepart::getOrgCode);
        if (oConvertUtils.isNotEmpty(id)) {
            String arr[] = id.split(",");
            query.in(SysDepart::getId, arr);
        }
        List<SysDepart> ls = this.sysDepartService.list(query);
        result.setSuccess(true);
        result.setResult(ls);
        return result;
    }

    /**
     * 查询数据 查出所有部门,并以树结构数据格式响应给前端
     *
     * @return
     */

    @RequestMapping(value = "/queryTreeByKeyWord", method = RequestMethod.GET)
    public Result<Map<String, Object>> queryTreeByKeyWord(@RequestParam(name = "keyWord", required = false) String keyWord) {
        Result<Map<String, Object>> result = new Result<>();
        try {
            Map<String, Object> map = new HashMap<String, Object>();
            List<SysDepartTreeModel> list = sysDepartService.queryTreeByKeyWord(keyWord);
            //根据keyWord获取用户信息
            LambdaQueryWrapper<SysUser> queryUser = new LambdaQueryWrapper<SysUser>();
            queryUser.eq(SysUser::getDelFlag, CommonConstant.DEL_FLAG_0);
            queryUser.and(i -> i.like(SysUser::getUsername, keyWord).or().like(SysUser::getRealname, keyWord));
            List<SysUser> sysUsers = this.sysUserService.list(queryUser);
            map.put("userList", sysUsers);
            map.put("departList", list);
            result.setResult(map);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }

    /**
     * 根据部门编码获取部门信息
     *
     * @param orgCode
     * @return
     */
    @GetMapping("/getDepartName")
    public Result<SysDepart> getDepartName(@RequestParam(name = "orgCode") String orgCode) {
        Result<SysDepart> result = new Result<>();
        LambdaQueryWrapper<SysDepart> query = new LambdaQueryWrapper<>();
        query.eq(SysDepart::getOrgCode, orgCode);
        SysDepart sysDepart = sysDepartService.getOne(query);
        result.setSuccess(true);
        result.setResult(sysDepart);
        return result;
    }

    /**
     * 根据部门id获取用户信息
     *
     * @param id
     * @return
     */
    @GetMapping("/getUsersByDepartId")
    public Result<List<SysUser>> getUsersByDepartId(@RequestParam(name = "id") String id) {
        Result<List<SysUser>> result = new Result<>();
        List<SysUser> sysUsers = sysUserDepartService.queryUserByDepId(id);

        result.setSuccess(true);
        result.setResult(sysUsers);
        return result;
    }

    /**
     * 通过下拉搜索获取数据
     */
    @ApiOperation(value = "下拉搜索部门接口", notes = "搜索部门信息")
    @RequestMapping(value = "/findDept", method = RequestMethod.GET)
    public Result<?> findArea(@ApiParam(name = "areaCode", value = "区划编码", defaultValue = "") @RequestParam(value = "areaCode", defaultValue = "") String areaCode,@ApiParam(name = "deptName", value = "部门名称", defaultValue = "") @RequestParam(value = "deptName", defaultValue = "") String deptName, @ApiParam(name = "limit", value = "限制多少条", defaultValue = "10") @RequestParam(value = "limit", defaultValue = "10") String limit) {
        QueryWrapper<SysDepart> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isEmpty(limit)) {
            queryWrapper.last(" limit 10");
        } else {
            int i = Integer.parseInt(limit);
            i = i > 100 ? 100 : i;
            queryWrapper.last(" limit " + i);
        }
        if (!StringUtils.isEmpty(deptName)) {
            queryWrapper.like("depart_name", deptName);
        }
        if (!StringUtils.isEmpty(areaCode)) {
            queryWrapper.eq("area_code", areaCode);
        }

        List<SysDepart> list = sysDepartService.list(queryWrapper);
        return Result.OK(list);
    }
}
